package resultsparser;

import au.com.bytecode.opencsv.CSVReader;
import au.com.bytecode.opencsv.CSVWriter;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.math.BigDecimal;
import java.nio.file.FileVisitResult;
import java.nio.file.Files;
import java.nio.file.Path;
import java.nio.file.SimpleFileVisitor;
import java.nio.file.attribute.BasicFileAttributes;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Comparator;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

//trida pro parsovani vysledku
public class ResultsParser {
    
    directoryListing dL = new directoryListing();
    ArrayList c, files;
    HashMap results;
    CSVReader reader, readerorig;

    public ResultsParser() throws FileNotFoundException, IOException {
        results = new HashMap();
        files = new ArrayList();
        loadFilesAll();
        getMaxMin();
        printMaxMinToCSV();
        sortCSV();
    }
    
    //pomocna trida pro porovnavani dvourozmernych stringu
    class ArrayComparator implements Comparator<Comparable[]> {
        private final int columnToSort;
        private final boolean ascending;

        public ArrayComparator(int columnToSort, boolean ascending) {
            this.columnToSort = columnToSort;
            this.ascending = ascending;
        }

        @Override
        public int compare(Comparable[] c1, Comparable[] c2) {            
            int cmp = c1[columnToSort].compareTo(c2[columnToSort]);
            return ascending ? cmp : -cmp;
        }
    }
    
    //trida zajistujici mazani slozek
    class sfv extends SimpleFileVisitor<Path> {
        
        public sfv() { }
        
        @Override
        public FileVisitResult visitFile(Path file, BasicFileAttributes attrs) throws IOException
        {
            Files.deleteIfExists(file);
            return FileVisitResult.CONTINUE;
        }
        @Override
        public FileVisitResult postVisitDirectory(Path dir, IOException e) throws IOException {
            if (e == null) {
                Files.deleteIfExists(dir);
                return FileVisitResult.CONTINUE;
            } else { throw e; }
        }
    }
    
    //funkce pro vytvoreni seznamu vsech zpracovavanych souboru
    private void loadFilesAll() {
        dL.listAll(new File(System.getProperty("user.dir") + "\\data\\")); 
        c = dL.listingAll;
        System.out.println("Nactene soubory:");
        for (int i = 0; i < dL.listingAll.size(); i++) {
            System.out.println(c.get(i));
        }
        System.out.println("");
    }
    
    //funkce pro zjisteni maxim a minim z nactenych vysledku
    private void getMaxMin() throws FileNotFoundException, IOException {        
        for (int i = 0; i < c.size(); i++) {
            boolean first = true;
            reader = new CSVReader(new FileReader(c.get(i).toString()),';');
            String [] line;
            while ((line = reader.readNext()) != null) {
                if (!first)
                    parseLine(line);
                first = false;
            }
        }
    }
    
    //funkce zpracovavajici jeden radek vysledku
    private void parseLine(String line[]) throws FileNotFoundException, IOException {
        if (line[1].indexOf("blif") == -1) {
            String key = line[0] + ";" + line[1] + ";" + line[2].substring(0, line[2].indexOf(".blif"));
            if (results.get(key) != null) {
                String entry[] = (String[])results.get(key);
                if (Integer.parseInt(line[3]) < Integer.parseInt(entry[3])) {
                    entry[3] = line[3];
                    entry[5] = line[4];
                    results.put(key, entry);
                }
                if (Integer.parseInt(line[3]) > Integer.parseInt(entry[4])) {
                    entry[4] = line[3];
                    entry[6] = line[4];
                    results.put(key, entry);
                }
                countAverages(entry, line[3]);
                results.put(key, entry);
            } else {
                String orig = getOrigStatString(line, getCommand(line[0]));
                String entry[] = {line[0], line[1], line[2].substring(0, line[2].indexOf(".blif")), line[3], line[3], line[4], line[4], orig, "0", "0", "0", "0"};
                countAverages(entry, line[3]);
                results.put(key, entry);
            }
        } else {
            String tempLine[] = new String[17];
            for (int i = 0; i < 4; i++) {               
                if (i == 1) {
                    tempLine[i] = "NP";
                    tempLine[i + 1] = line[i];
                    continue;
                }
                if (i >= 2) {
                    tempLine[i + 1] = line[i];
                    continue;
                }
                tempLine[i] = line[i];
            }
            String key = tempLine[0] + ";" + tempLine[1] + ";" + tempLine[2].substring(0, tempLine[2].indexOf(".blif"));
            if (results.get(key) != null) {
                String entry[] = (String[])results.get(key);
                if (Integer.parseInt(tempLine[3]) < Integer.parseInt(entry[3])) {
                    entry[3] = tempLine[3];
                    entry[5] = tempLine[4];
                    results.put(key, entry);
                }
                if (Integer.parseInt(tempLine[3]) > Integer.parseInt(entry[4])) {
                    entry[4] = tempLine[3];
                    entry[6] = tempLine[4];
                    results.put(key, entry);
                }
                countAverages(entry, tempLine[3]);
                results.put(key, entry);
            } else {
                String orig = getOrigStatString(tempLine, getCommand(tempLine[0]));
                String entry[] = {tempLine[0], tempLine[1], tempLine[2].substring(0, tempLine[2].indexOf(".blif")), tempLine[3], tempLine[3], tempLine[4], tempLine[4], orig, "0", "0", "0", "0"};
                countAverages(entry, tempLine[3]);
                results.put(key, entry);
            }
        }       
    }
    
    //funkce pro vypocet prumeru
    private String[] countAverages(String[] entry, String and) {
        entry[8] = String.valueOf(Integer.valueOf(entry[8]) + 1);
        if (Integer.parseInt(and) < Integer.parseInt(entry[7])) {
            entry[9] = String.valueOf(Integer.valueOf(entry[9]) + 1);            
        }
        if (Integer.parseInt(and) > Integer.parseInt(entry[7])) {
            entry[10] = String.valueOf(Integer.valueOf(entry[10]) + 1);            
        }
        BigDecimal p1 = new BigDecimal(String.valueOf(((Double.valueOf(and) / Double.valueOf(entry[7])) * 100) - 100));        
        p1 = p1.setScale(2, BigDecimal.ROUND_HALF_UP);
        BigDecimal p2 = new BigDecimal(String.valueOf(entry[11]));        
        p2 = p2.setScale(2, BigDecimal.ROUND_HALF_UP);
        BigDecimal p3 = p2.add(p1);        
        p3 = p3.setScale(2, BigDecimal.ROUND_HALF_UP);
        entry[11] = String.valueOf(p3);
        return entry;
    }
    
    //funkce pro vypocet absolutnich a relativnich rozdilu
    private String[] computePercDiff(String [] line) {
        String[] lineP = new String[line.length + 1];
        BigDecimal p = new BigDecimal(String.valueOf(((Double.valueOf(line[3]) / Double.valueOf(line[1])) * 100) - 100));        
        p = p.setScale(2, BigDecimal.ROUND_HALF_UP);
        BigDecimal pmin = new BigDecimal(String.valueOf((Double.valueOf(line[3]) / Double.valueOf(line[1])) * 100));        
        pmin = pmin.setScale(2, BigDecimal.ROUND_HALF_UP);
        BigDecimal pmax = new BigDecimal(String.valueOf((Double.valueOf(line[4]) / Double.valueOf(line[1])) * 100));        
        pmax = pmax.setScale(2, BigDecimal.ROUND_HALF_UP);
        System.arraycopy(line, 0, lineP, 0, 13);
        lineP[5] = String.valueOf(pmin);
        lineP[6] = String.valueOf(pmax);
        lineP[7] = line[7];
        lineP[lineP.length - 1] = String.valueOf(p);
        return lineP;
    }
    
    //funkce pro zapis do formatu CSV s pridanim originalnich vysledku, maxim a minim
    private void printMaxMinToCSV() throws IOException {
        String result[];
        String resultWithDif[];
        String command;
        FileWriter fileWriter;
        CSVWriter writer;
        Set entries = results.entrySet();
        Iterator iter = entries.iterator();
        while (iter.hasNext()) {
            Map.Entry entry = (Map.Entry) iter.next();
            result = (String[]) entry.getValue();
            resultWithDif = new String[result.length - 1];
            System.arraycopy(result, 2, resultWithDif, 0, result.length - 2);
            resultWithDif[resultWithDif.length - 1] = String.valueOf(Integer.parseInt(result[4]) - Integer.parseInt(result[3]));
            command = getCommand(result[0]);
            resultWithDif = addOrigStat(resultWithDif, command);
            String[] resultWithPerc = computePercDiff(resultWithDif);
            resultWithPerc = prepareTable(resultWithPerc);
            fileWriter = new FileWriter(System.getProperty("user.dir") + "\\results\\" + result[0] + result[1] + ".csv", true);
            if (!files.contains(System.getProperty("user.dir") + "\\results\\" + result[0] + result[1] + ".csv"))
                files.add(System.getProperty("user.dir") + "\\results\\" + result[0] + result[1] + ".csv");
            writer = new CSVWriter(fileWriter, ';');            
            writer.writeNext(resultWithPerc);
            fileWriter.close();
            writer.close();           
        }
        System.out.println("Zapis Dokoncen");
        System.out.println("");
    }
    
    //pomocna funkce pro urceni prikazu pri hledani originalnich vysledku
    private String getCommand(String c) {
        String command = "";
        if (c.indexOf("balance") != -1) {
            command = "balance";
            return command;
        }
        if (c.indexOf("rewrite") != -1) {
            command = "rewrite";
            return command;
        }
        if (c.indexOf("refactor") != -1) {
            command = "refactor";
            return command;
        }
        if (c.indexOf("resub") != -1) {
            command = "resub";
            return command;
        }
        if (c.indexOf("rr") != -1) {
            command = "rr";
            return command;
        }
        if (c.indexOf("collapse") != -1) {
            command = "collapse";
            return command;
        }
        return command;
    }
    
    //funkce pro zaverecny dopocet hodnot do tabulky a jejich preusporadani
    private String[] prepareTable(String[] line) {
        String[] lineR = new String[line.length + 3];
        System.arraycopy(line, 0, lineR, 0, 7);
        lineR[7] = line[12];        
        lineR[8] = line[0];
        lineR[9] = line[1];
        lineR[10] = line[2];
        BigDecimal p;
        p = new BigDecimal(String.valueOf(((Double.valueOf(line[1]) - Double.valueOf(line[3])) / Double.valueOf(line[1])) * 100));        
        p = p.setScale(2, BigDecimal.ROUND_HALF_UP);
        lineR[11] = String.valueOf(p);
        p = new BigDecimal(String.valueOf(((Double.valueOf(line[1]) - Double.valueOf(line[4])) / Double.valueOf(line[1])) * 100));        
        p = p.setScale(2, BigDecimal.ROUND_HALF_UP);
        lineR[12] = String.valueOf(p);
        p = new BigDecimal(String.valueOf((Double.valueOf(line[9]) / Double.valueOf(line[8])) * 100));        
        p = p.setScale(2, BigDecimal.ROUND_HALF_UP);
        lineR[13] = String.valueOf(p);
        p = new BigDecimal(String.valueOf((Double.valueOf(line[10]) / Double.valueOf(line[8])) * 100));        
        p = p.setScale(2, BigDecimal.ROUND_HALF_UP);
        lineR[14] = String.valueOf(p);
        p = new BigDecimal(String.valueOf(Double.valueOf(line[11]) / Double.valueOf(line[8])));        
        p = p.setScale(2, BigDecimal.ROUND_HALF_UP);
        lineR[15] = String.valueOf(p);
        lineR[16] = line[13];
        return lineR;
    }
    
    //funkce pro nacteni originalnich vysledku
    private String[] addOrigStat(String[] line, String command) throws FileNotFoundException, IOException {
        String[] rLine = new String[line.length + 2];
        System.arraycopy(line, 0, rLine, 0, 1);
        String orig[] = getOrigStat(line, command);
        rLine[1] = orig[0];
        rLine[2] = orig[1];
        System.arraycopy(line, 1, rLine, 3, 10);
        return rLine;
    }
    
    //funkce pro zjisteni originalnich vysledku ze souboru s nimi - pouze AND
    private String getOrigStatString(String[] line, String command) throws FileNotFoundException, IOException {
        readerorig = new CSVReader(new FileReader(System.getProperty("user.dir") + "\\circuitsorigStats\\origStats.csv"),';');
        String [] origLine;
        String result = "";
        boolean first = true;
        while ((origLine = readerorig.readNext()) != null) {
            if (!first) {
                if (origLine[1].indexOf("blif") == -1) {
                    if(line[0].equals(origLine[2].substring(0, origLine[2].indexOf(".blif"))) && command.equals(origLine[0])) {
                        result = origLine[3];
                        break;
                    }
                } else {
                    if (line[2].indexOf("blif") != -1) {
                        if(line[2].substring(0, line[2].indexOf(".blif")).equals(origLine[1].substring(0, origLine[1].indexOf(".blif"))) && command.equals(origLine[0])) {
                            result = origLine[2];
                            break;
                        }
                    } else {
                        if(line[1].substring(0, line[1].indexOf(".blif")).equals(origLine[1].substring(0, origLine[1].indexOf(".blif"))) && command.equals(origLine[0])) {
                            result = origLine[2];
                            break;
                        }
                    }                   
                }
            }
            first = false;
        }
        return result;
    }
    
    //funkce pro zjisteni originalnich vysledku ze souboru s nimi
    private String[] getOrigStat(String[] line, String command) throws FileNotFoundException, IOException {
        reader = new CSVReader(new FileReader(System.getProperty("user.dir") + "\\circuitsorigStats\\origStats.csv"),';');
        String [] origLine;
        String [] result = new String[2];
        boolean first = true;
        while ((origLine = reader.readNext()) != null) {
            if (!first) {
                if (origLine[1].indexOf("blif") == -1) {
                    if(line[0].equals(origLine[2].substring(0, origLine[2].indexOf(".blif"))) && command.equals(origLine[0])) {
                        result[0] = origLine[3];
                        result[1] = origLine[4];
                        break;
                    }
                } else {
                    if(line[0].equals(origLine[1].substring(0, origLine[1].indexOf(".blif"))) && command.equals(origLine[0])) {
                        result[0] = origLine[2];
                        result[1] = origLine[3];
                        break;
                    }
                }
            }
            first = false;
        }
        return result;
    }
    
    //funkce pro serazeni vysledku pro tabulky dle rozdilu mezi maximem a minimem
    private void sortCSV() throws FileNotFoundException, IOException {
        ArrayList temp;
        ArrayList temp2;
        FileWriter fileWriter;
        CSVWriter writer;
        for (int i = 0; i < files.size(); i++) {
            reader = new CSVReader(new FileReader(files.get(i).toString()),';');
            String [] line;
            String lineString = "";
            temp = new ArrayList();
            temp2 = new ArrayList();
            while ((line = reader.readNext()) != null) {
                lineString = "";
                for (int j = 0; j < line.length; j++) {
                    lineString += line[j] + ";";
                }
                lineString = lineString.substring(0, lineString.length() - 1);
                temp.add(lineString);
                temp2.add(line);
            }
            Double [][] sortLines = new Double[temp.size()][2];
            for (int j = 0; j < temp.size(); j++) {
                sortLines[j][0] = Double.valueOf(temp.get(j).toString().substring(temp.get(j).toString().lastIndexOf(";") + 1));
                sortLines[j][1] = (double)j;
            }
            Arrays.sort(sortLines, new ArrayComparator(0, true));
            String[] resultOrderToWrite = new String[temp.size()];
            for (int j = 0; j < temp.size(); j++) {
                resultOrderToWrite[j] = String.valueOf(sortLines[j][1]);
            }
            writer = new CSVWriter(new FileWriter(files.get(i).toString().substring(0,files.get(i).toString().lastIndexOf(".")) + "sorted.csv"), ';');
            if (files.get(i).toString().indexOf("collapse") == -1) {
                String[] header = {"Obvod", "oA", "oL", "minA", "maxA", "min%", "max%", "d", "&", "Obvod", "oA", "oL", "minR%", "maxR%", "prum+", "prum-", "prumC"};
                writer.writeNext(header);
                writer.close();
            } else {
                String[] header = {"Obvod", "oB", "oL", "minB", "maxB", "min%", "max%", "d", "&", "Obvod", "oB", "oL", "minR%", "maxR%", "prum+", "prum-", "prumC"};
                writer.writeNext(header);
                writer.close();
            }            
            
            String[] resultToWrite;
            String[] resultToWriteS;
            for (int j = 0; j < temp2.size(); j++) {
                resultToWrite = (String [])temp2.get((int)Double.parseDouble(resultOrderToWrite[j]));
                resultToWriteS = new String[resultToWrite.length];
                System.arraycopy(resultToWrite, 0, resultToWriteS, 0, 8);
                resultToWriteS[8] = resultToWrite[16];
                System.arraycopy(resultToWrite, 8, resultToWriteS, 9, 8);
                fileWriter = new FileWriter(files.get(i).toString().substring(0,files.get(i).toString().lastIndexOf(".")) + "sorted.csv", true);
                writer = new CSVWriter(fileWriter, ';');            
                writer.writeNext(resultToWriteS);
                fileWriter.close();
                writer.close();
            }            
        }
        System.out.println("Sort dokoncen");
        System.out.println("");
    }
    
    //spousteni aplikace resultsParser
    public static void main(String[] args) throws IOException {			
        new ResultsParser();
	System.out.println("Hotovo");		
    }
}

